'use strict';

const _ = require('lodash');
const common = require('../common');
const dataTypes = require('../definitions/datatypes');

const options = [
  {
    flags: '-d, --db <name>',
    description: 'The database to use for the query',
    autocomplete: common.autoCompleteDatabases
  },
  {
    flags: '--drop',
    description: 'Drops the table before creating it.'
  }
];

/**
 * Get the column informations optionally using previous answers
 * @param  {Object} previous The previous answers object
 * @return {Array}           An array of questions object
 */
const columnQuestions = (previous) => {
  return [
    {
      type: 'input',
      name: 'name',
      message: 'Enter the column name: '
    }, {
      type: 'list',
      name: 'datatype',
      message: 'Enter the column data type: ',
      default: previous.datatype || 0,
      choices: dataTypes
    }, {
      type: 'input',
      name: 'datalength',
      message: 'Enter the column length: ',
      default: 50,
      validate: v => (v >= 0 && v <= 255)
        ? true
        : 'The length must be any value from 0 to 255.',
      when: answers => _.find(dataTypes, {value: answers.datatype}).size
    }
  ];
};

/**
 * From the columns array build the columns query string
 * @param  {Array} columns  The columns objects array
 * @return {String}         A string to use in MySQL query
 */
const getColumnsString = columns => {
  return columns.map(col => {
    const type = col.datalength
      ? `${col.datatype}(${col.datalength})`
      : `${col.datatype}`;
    return `${col.name} ${type}`
  }).join(',');
};

module.exports = {
  name: 'create table <name>',
  description: 'Create a new table.',
  options: options,
  before: common.validateConnection,
  action: function(args, cb) {
    const {appData, chalk} = this.parent;
    const {name, options} = args;
    const database = options.db || appData.currentDatabase;
    const columns = [];

    this.log('');

    // First ask how many columns will the table have
    return this.prompt({
      type: 'input',
      name: 'number',
      message: 'How many columns does it have? ',
      filter: v => parseInt(v)
    }, answers => {
      const {number} = answers;
      let previousAnswers = {};

      this.log('');

      // Create an array of N elements to collect column informations
      // using built-in questions system and aggregating answers in array
      new Array(number).fill().reduce(prev => {
        return prev.then(() => {
          return this.prompt(columnQuestions(previousAnswers)).then(answers => {
            this.log('');
            previousAnswers = answers;
            columns.push(answers);
          });
        });
      }, Promise.resolve()).then(() => {
        let query = `CREATE TABLE ${name} (${getColumnsString(columns)})`;
        if (options.drop) query = `DROP TABLE IF EXISTS ${name}; ${query}`

        this.parent.query(query, options.db, cb, data => {
          appData.databases[database].tables.push(name);
          this.log(`The table ${chalk.green(name)} has been created successfully.\n`);
          cb(null, data);
        });
      });
    });
  }
};
