"use strict";

const fs = require("fs");
const path = require("path");
const updateNotifier = require('update-notifier');
const pkg = require("../package.json");
const banner = require("./banner");
const Vorpal = require("vorpal").default;

// Check for updates
updateNotifier({pkg}).notify({
  isGlobal: true
});

// Create the program instance
const program = new Vorpal({debug: false});

/**
 * Export the application
 */
module.exports = program;

// Get all commands
const commandsPath = path.join(__dirname, "commands");
const commands = fs.readdirSync(commandsPath).map(f => path.join(commandsPath, f));

// Load commands
commands.forEach(c => program.use(c));

// Set the variable to hold MySQL connection
program.conn = null;
program.appData = {
  currentDatabase: null,
  currentTable: null,
  databases: {}
};

// Set the app delimiter based on appData session variables
program.refreshDelimiter = function() {
  const {appData, chalk} = this;
  const main = appData.currentHost
    ? [
      chalk.bold(appData.currentUser),
      chalk.yellow(`@${appData.currentHost}`),
      appData.currentPort !== 3306
        ? `:${appData.currentPort}`
        : ''
    ].join('') : pkg.name;
  let str = [`[${main}]`];
  if (appData.currentDatabase)
    str.push(`[${chalk.green(appData.currentDatabase)}]`)
  this.delimiter(str.join(''));
};

/**
 * Query the connected MySQL database and handle response
 * @param  {String}   query The query to run
 * @param  {String}   db    The database to use
 * @param  {Function}   errCb The error callback
 * @param  {Function} cb    The response callback
 */
program.query = function(query, db, errCb, cb) {
  const {currentDatabase} = this.appData;
  const previous = currentDatabase;

  // If database exists and is not current one selected
  if (db && db !== previous) {

    // If there is no current database select that one
    if (!previous) {
      this.useDatabase(db).then(() => {
        this.query(query, null, errCb, cb);
      }).catch(errCb);
      return;
    }

    // Run a multiple statement query switching database
    query = [`USE ${db}`, query, `USE ${previous}`, ''].join('; ');
  }

  // Print the query to console as debug
  if (this._options.debug) {
    this.log(query);
  }

  // Run the query
  program.ui.redraw('\n Loading...  \n');
  this.conn.query({
    sql: query,
    timeout: 30000
  }, function(err, results) {
    program.ui.redraw.clear();
    if (err) {
      return errCb(`\n${err.message}\n`);
    }
    cb(results);
  });
};

/**
 * Query the databases parse the response, update the save data
 * than resolve with the raw response and the parsed data
 * @return {Promise} A promise resolved with an object
 */
program.listDatabases = function() {
  return new Promise((resolve, reject) => {
    this.query('SHOW DATABASES', null, reject, data => {
      const databases = data.map(d => d.Database);
      this.appData.databases = databases.reduce((obj, d) => {
        obj[d] = this.appData.databases[d] || {
          tables: []
        };
        return obj;
      }, {});
      resolve({raw: data, list: databases});
    });
  });
};

/**
 * Select the database, update the session variables
 * and call the delimiter update to reflect the changes on UI
 * @param  {String} db The name of the database to select
 * @return {Promise}   A promise resolved when database is selected
 */
program.useDatabase = function(db) {
  return new Promise((resolve, reject) => {
    if (!db || db === '') {
      this.appData.currentDatabase = null;
      this.refreshDelimiter();
      resolve();
      return;
    }
    this.query(`USE ${db}`, null, reject, () => {
      this.appData.currentDatabase = db;
      this.appData.databases[db] = this.appData.databases[db] || {};
      this.refreshDelimiter();
      resolve();
    });
  });
};

// Setup the application
program
  .title(pkg.name)
  .description(pkg.description)
  .version(pkg.version)
  .delimiter(`[${pkg.name}]`)
  .history(pkg.name)
  .localStorage(pkg.name)
  .banner(banner)
  .onExit(function() {
    const self = this;
    return new Promise(resolve => {
      if (!self.conn) {
        resolve();
        return;
      }

      self.log("Closing the connection.");
      self.conn.end(function() {
        resolve();
      });
    });
  });

// Start interactive command line
program.show();
